﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Sirenix.OdinInspector;

public class DataBase_Manager : MonoBehaviour
{
    public static DataBase_Manager Instance;

    [SerializeField] private CharacterData[] charDataArr;
    [SerializeField] private StageData[] stageDataArr;
    [SerializeField] private BgData[] bgDataArr;
    private Dictionary<CharacterType, CharacterData> charDataDic;

    public TestData testData;

    public void Init_Func()
    {
        Instance = this;

        charDataDic = new Dictionary<CharacterType, CharacterData>();

        foreach (var _charData in charDataArr)
        {
            CharacterType _charType = _charData.charClass.CharacterType;

            this.charDataDic.Add_Func(_charType, _charData);
        }
    }

    public Character_Script GetCharClass_Func(CharacterType _charType)
    {
        CharacterData _charData = this.charDataDic.GetValue_Func(_charType);
        return Instantiate(_charData.charClass);
    }
    public Sprite GetCharSprite_Func(CharacterType _charType)
    {
        CharacterData _charData = this.charDataDic.GetValue_Func(_charType);
        return _charData.charClass.CharSprite;
    }

    public StageData GetStageData_Func(int _stageId)
    {
        return this.stageDataArr[_stageId - 1];
    }

    public int GetLastStage_Func()
    {
        return stageDataArr.Length;
    }

    public BgData GetBgData_Func(BgType _bgType)
    {
        return this.bgDataArr[(int)_bgType - 1];
    }

    [System.Serializable]
    public class TestData
    {
        public bool isTestOn;

        [EnableIf("isTestOn")] public int startStageId;
    }
}

[System.Serializable]
public class CharacterData
{
    public Character_Script charClass;
}

[System.Serializable]
public class StageData
{
    [SerializeField] private int stageId;
    public int healthPoint;
    public BgType bgType;

    public StageCharData[] stageCharDataArr;
    
    public StageCharData GetStageCharData_Func(CharacterType _charType)
    {
        StageCharData _data = null;

        foreach (StageCharData item in this.stageCharDataArr)
        {
            if(_charType == item.charType)
            {
                _data = item;
                break;
            }
        }

        return _data;
    }

    [System.Serializable]
    public class StageCharData
    {
        public bool isLiar;
        public CharacterType charType;
        [TextArea(0, 2)] public string speech;
    }
}

[System.Serializable]
public class BgData
{
    public BgType bgType;
    public Sprite offSprite;
    public Sprite onSprite;
}