﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening;

public class SoundSystem_Manager : MonoBehaviour
{
    public static SoundSystem_Manager Instance;

    [SerializeField] private AudioSource bgmAS;
    [SerializeField] private AudioSource sfxAS;
    [SerializeField] private AudioClip victroyBgm;

    [SerializeField] private SoundData[] soundDataArr;
    private Dictionary<SfxType, SoundData> soundDataDic;
    private Coroutine volumnCor;

    public void Init_Func()
    {
        Instance = this;

        soundDataDic = new Dictionary<SfxType, SoundData>();
        foreach (SoundData _soundData in this.soundDataArr)
            this.soundDataDic.Add_Func(_soundData.sfxType, _soundData);

        bgmAS.Play();
    }

    public void PlaySfx_Func(SfxType _sfxType)
    {
        SoundData _soundData = soundDataDic.GetValue_Func(_sfxType);
        sfxAS.PlayOneShot(_soundData.audioClip);
    }

    public void PlayVictoryBgm_Func()
    {
        bgmAS.clip = this.victroyBgm;
        bgmAS.Play();
    }

    public void SetBgmVolumn_Func(float _targetVolumn, float _time = 0f)
    {
        if(0f < _time)
        {
            volumnCor = StartCoroutine(SetBgmVolumn_Cor(_targetVolumn, _time));
        }
        else
        {
            this.bgmAS.volume = _targetVolumn;
        }
    }
    private IEnumerator SetBgmVolumn_Cor(float _targetVolumn, float _time)
    {
        float _changeValue = 1f - _targetVolumn;

        float _calcTime = 0f;
        while(_calcTime < _time)
        {
            _calcTime += Time.deltaTime;

            float _value = (_changeValue * (_calcTime / _time));
            this.bgmAS.volume = 1f - _value;

            yield return null;
        }

        this.bgmAS.volume = 1f - _changeValue;

        yield break;
    }

    [System.Serializable]
    public class SoundData
    {
        public SfxType sfxType;
        public AudioClip audioClip;
    }

    public enum SfxType
    {
        None = 0,

        GunShot,
        Normal,
        Select,
        News,
        NextDay,
    }
}
