﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using Cargold.WhichOne;

public class StageSystem_Manager : MonoBehaviour
{
    public static StageSystem_Manager Instance;

    [SerializeField] private Transform charLeftPosTrf;
    [SerializeField] private Transform charRightPosTrf;
    [SerializeField] private SpriteRenderer offBgSrnd;
    [SerializeField] private SpriteRenderer onBgSrnd;
    [SerializeField] private Animation bgAnim;
    private List<Character_Script> characterList;
    private List<Character_Script> peopleCharacterList;
    private List<Character_Script> liarCharacterList;
    private int stageId;
    private int hp;
    private int liarCount;
    private WhichOne<Character_Script> characterWhichOne;

    public int StageId { get => stageId; }

    public void Init_Func()
    {
        Instance = this;

        stageId = 0;

        characterList = new List<Character_Script>();
        peopleCharacterList = new List<Character_Script>();
        liarCharacterList = new List<Character_Script>();

        characterWhichOne = new WhichOne<Character_Script>();
    }

    public void StartStage_Func(int _stageId)
    {
        SoundSystem_Manager.Instance.SetBgmVolumn_Func(1f, 0.5f);

        foreach (var _character in this.characterList)
            _character.Deactivate_Func();

        this.characterList.Clear();
        this.peopleCharacterList.Clear();
        this.liarCharacterList.Clear();

        this.stageId = _stageId;
        
        StageData _stageData = DataBase_Manager.Instance.GetStageData_Func(_stageId);
        this.hp = _stageData.healthPoint;

        BgData _bgData = DataBase_Manager.Instance.GetBgData_Func(_stageData.bgType);
        this.offBgSrnd.sprite = _bgData.offSprite;
        this.onBgSrnd.sprite = _bgData.onSprite;
        this.bgAnim.Play();

        float _charPosSpaceMax = charRightPosTrf.position.x + Mathf.Abs(this.charLeftPosTrf.position.x);
        int _stageCharNum = _stageData.stageCharDataArr.Length;
        float _charPosSpaceGap = _charPosSpaceMax / (_stageCharNum + 1);
        float _charPosSpace = charLeftPosTrf.position.x;

        for (int i = 0; i < _stageCharNum; i++)
        {
            StageData.StageCharData _stageCharData = _stageData.stageCharDataArr[i];
            CharacterType _charType = _stageCharData.charType;

            Character_Script _charClass = DataBase_Manager.Instance.GetCharClass_Func(_charType);
            bool _isLiar = _stageCharData.isLiar;
            string _speech = _stageCharData.speech;
            _charPosSpace += _charPosSpaceGap;
            _charClass.Activate_Func(_isLiar, _speech, new Vector2(_charPosSpace, this.charLeftPosTrf.position.y));

            if (_isLiar == true)
                this.liarCharacterList.AddNewItem_Func(_charClass);
            else
                this.peopleCharacterList.AddNewItem_Func(_charClass);

            characterList.AddNewItem_Func(_charClass);
        }

        UI_Status_Manager.Instance.StageStart_Func(_stageId, this.hp, this.liarCharacterList.Count);
    }

    public void SelectCharClass_Func(Character_Script _charClass)
    {
        this.characterWhichOne.Selected_Func(_charClass, false);
    }

    public void Execute_Func()
    {
        if(this.characterWhichOne.HasWhichOne_Func() == true)
        {
            Character_Script _executeCharClass = this.characterWhichOne.GetWhichOne_Func();
            
            if(_executeCharClass.IsLiar == true)
            {
                this.liarCharacterList.Remove_Func(_executeCharClass);

                UI_Status_Manager.Instance.SetLiar_Func(this.liarCharacterList.Count);

                if(this.liarCharacterList.Count <= 0)
                {
                    this.bgAnim.Stop();

                    int _lastStage = DataBase_Manager.Instance.GetLastStage_Func();
                    if(this.stageId < _lastStage)
                    {
                        // Stage Clear

                        UI_SceneChange_Manager.Instance.StageClear_Func();
                    }
                    else
                    {
                        // Game Clear
                        
                        UI_Victory_Manager.Instance.OnVictory_Func();
                    }
                }
                else
                {
                    SoundSystem_Manager.Instance.SetBgmVolumn_Func(1f, 1f);
                }
            }
            else
            {
                this.peopleCharacterList.Remove_Func(_executeCharClass);

                this.hp--;

                UI_Status_Manager.Instance.SetHP_Func(this.hp);

                if(this.hp <= 0)
                {
                    // Game Over
                    
                    UI_SceneChange_Manager.Instance.StageOver_Func();
                }
                else
                {
                    SoundSystem_Manager.Instance.SetBgmVolumn_Func(1f, 1f);
                }
            }

            _executeCharClass.Kill_Func();
        }
        else
        {
            Debug_C.Error_Func("?");
        }
    }
}
