﻿using DG.Tweening;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using Sirenix.OdinInspector;

public class UI_DialogManager : MonoBehaviour
{
    public static UI_DialogManager Instance;

    [BoxGroup("공통")] [SerializeField] private GameObject groupObj;
    [BoxGroup("공통")] [SerializeField] private Image bgImg;
    [BoxGroup("공통")] [SerializeField] private Image charImg;

    [BoxGroup("대화")] [SerializeField] private GameObject dialogueGroupObj;
    [BoxGroup("대화")] [SerializeField] private GameObject executeBtnObj;
    [BoxGroup("대화")] [SerializeField] private Text nameTxt;
    [BoxGroup("대화")] [SerializeField] private Text dialogueTxt;
    [BoxGroup("대화")] [SerializeField] private RectTransform dialogueCharPosRtrf;

    [BoxGroup("처형")] [SerializeField] private GameObject executeGroupObj;
    [BoxGroup("처형")] [SerializeField] private Animation executeAnim;
    [BoxGroup("처형")] [SerializeField] private float executedFadeValue;
    [BoxGroup("처형")] [SerializeField] private float executedTime;

    private StepType stepType;

    public void Init_Func()
    {
        Instance = this;

        stepType = StepType.None;

        groupObj.SetActive(false);
    }

    public void Activate_Func(Character_Script _charClass)
    {
        int _stageId = StageSystem_Manager.Instance.StageId;
        StageData _stageData = DataBase_Manager.Instance.GetStageData_Func(_stageId);

        CharacterType _charType = _charClass.CharacterType;
        bool _isAlive = _charClass.IsAlive;
        StageData.StageCharData _stageCharData = _stageData.GetStageCharData_Func(_charType);
        if(_stageCharData != null)
        {
            groupObj.SetActive(true);
            dialogueGroupObj.SetActive(true);

            stepType = StepType.Dialogue;

            this.nameTxt.text = _charClass.CharName;
            this.dialogueTxt.text = _stageCharData.speech;

            Sprite _charSprite = _charClass.CharSprite;
            this.charImg.SetNativeSize_Func(_charSprite);
            this.charImg.color = _isAlive == true
                ? Color.white
                : new Color(0.25f, 0.25f, 0.25f, 1f);

            executeBtnObj.SetActive(_isAlive);

            this.bgImg.SetFade_Func(0.4f);

            this.charImg.rectTransform.position = dialogueCharPosRtrf.position;
            this.charImg.rectTransform.localScale = Vector3.one * 0.75f;

            SoundSystem_Manager.Instance.PlaySfx_Func(SoundSystem_Manager.SfxType.Select);
        }
        else
        {
            Debug_C.Error_Func("?");
        }
    }

    public void Deactivate_Func()
    {
        if (stepType == StepType.Dialogue)
        {
            dialogueGroupObj.SetActive(false);
        }
        else if (this.stepType == StepType.Execute)
        {
            this.executeGroupObj.SetActive(false);
            this.executeAnim.gameObject.SetActive(false);
        }
        else
        {
            Debug_C.Error_Func("?");
        }

        stepType = StepType.None;

        groupObj.SetActive(false);
    }

    public void CallBtn_Execute_Func()
    {
        if(stepType == StepType.Dialogue)
        {
            stepType = StepType.Execute;

            this.dialogueGroupObj.SetActive(false);
            this.executeGroupObj.SetActive(true);

            this.bgImg.DOFade(0.8f, 0.5f);
            this.charImg.rectTransform.DOLocalMoveX(0f, 1f);
            this.charImg.rectTransform.DOScale(1f, 1f);

            SoundSystem_Manager.Instance.PlaySfx_Func(SoundSystem_Manager.SfxType.Select);
        }
        else
        {
            Debug_C.Error_Func("?");
        }
    }
    public void CallBtn_Okay_Func()
    {
        SoundSystem_Manager.Instance.PlaySfx_Func(SoundSystem_Manager.SfxType.Normal);

        this.Deactivate_Func();
    }
    public void CallBtn_Execute_Confirm_Func()
    {
        if(this.stepType == StepType.Execute)
        {
            executeAnim.gameObject.SetActive(true);
            executeAnim.Play();

            SoundSystem_Manager.Instance.SetBgmVolumn_Func(0f, 0.5f);
        }
        else
        {
            Debug_C.Error_Func("?");
        }
    }
    public void CallAni_Func(string _key)
    {
        if(_key == "Kill")
        {
            charImg.DOColor(new Color(executedFadeValue, executedFadeValue, executedFadeValue, 1f), executedTime);

            SoundSystem_Manager.Instance.PlaySfx_Func(SoundSystem_Manager.SfxType.GunShot);
        }
        else if(_key == "Done")
        {
            StageSystem_Manager.Instance.Execute_Func();

            Deactivate_Func();
        }
    }

    public enum StepType
    {
        None = 0,

        Dialogue,
        Execute,
    }
}
