﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using DG.Tweening;
using TMPro;

public class UI_Intro_Manager : MonoBehaviour
{
    public static UI_Intro_Manager Instance;

    [SerializeField] private GameObject groupObj;
    [SerializeField] private GameObject introBgObj;
    [SerializeField] private Image bgImg;
    [SerializeField] private TextMeshProUGUI[] introTmpArr;
    [SerializeField] private Image titleImg;
    [SerializeField] private Text touchTxt;

    private StepType stepType;
    private Sequence touchBlinkSeq;

    public void Init_Func()
    {
        Instance = this;

        stepType = StepType.None;

        touchBlinkSeq = DOTween.Sequence();
        touchBlinkSeq.Append(touchTxt.DOFade(0f, 1f));
        touchBlinkSeq.Append(touchTxt.DOFade(1f, 1f));
        touchBlinkSeq.SetLoops(-1);
        touchBlinkSeq.Pause();

        //bgImg.SetFade_Func(1f);
        foreach (var _introTmp in introTmpArr)
            _introTmp.SetFade_Func(0f);
        titleImg.SetFade_Func(0f);
        touchTxt.SetFade_Func(0f);

        groupObj.SetActive(true);
    }

    public void Start_Func()
    {
        groupObj.SetActive(true);

        StartCoroutine(Start_Cor());
    }
    private IEnumerator Start_Cor()
    {
        bgImg.DOFade(0f, 3f);

        foreach (var _introTmp in introTmpArr)
        {
            _introTmp.DOFade(1f, 1f);
            yield return Coroutine_C.GetWaitForSeconds_Cor(1f);
        }

        this.stepType = StepType.IntroBgFadeDone;
        
        yield break;
    }
    private IEnumerator CameraMove_Cor()
    {
        foreach (var _introTmp in introTmpArr)
        {
            _introTmp.DOFade(0f, 1f);
        }

        CameraSystem_Manager.Instance.Move_Func(Vector2.zero, 1f);

        yield return Coroutine_C.GetWaitForSeconds_Cor(1f);

        stepType = StepType.TitleShow;

        //bgImg.DOFade(0.5f, 1f);
        Tween _twn = titleImg.DOFade(1f, 1f);
        //touchTxt.DOFade(1f, 1f);

        yield return _twn.WaitForCompletion();

        stepType = StepType.TitleBlink;

        touchBlinkSeq.Play();

        yield break;
    }
    private IEnumerator GameStart_Cor()
    {
        touchBlinkSeq.Kill();
        touchTxt.DOFade(0f, 1f);
        bgImg.DOFade(1f, 1f);
        Tween _twn = titleImg.DOFade(0f, 1f);

        yield return _twn.WaitForCompletion();

        this.stepType = StepType.GameStart;

        int _startStageId = 1;
        if (DataBase_Manager.Instance.testData.isTestOn == true)
            _startStageId = DataBase_Manager.Instance.testData.startStageId;

        UI_SceneChange_Manager.Instance.StartStage_Func(_startStageId);

        GameObject.Destroy(introBgObj);

        groupObj.SetActive(false);

        yield break;
    }
    
    public void CallBtn_Func()
    {
        if (stepType == StepType.IntroBgFadeDone)
        {
            Debug_C.Log_Func("CameraWorking");

            stepType = StepType.CameraWorking;

            StartCoroutine(CameraMove_Cor());
        }
        else if (stepType == StepType.TitleBlink)
        {
            Debug_C.Log_Func("TitleFadeOut");

            stepType = StepType.TitleFadeOut;

            StartCoroutine(GameStart_Cor());
        }
    }

    private enum StepType
    {
        None = 0,

        IntroBgFadeDone,
        CameraWorking,
        TitleShow,
        TitleBlink,
        TitleFadeOut,
        GameStart,
    }
}
